#!/bin/bash

# ==========
# $File: //guest/amo/scripts/speccount.sh $
# $Revision: #2 $
# $Change: 24799 $
#
# $Author: amo $
# $DateTimeTZ: 2018/10/25 07:48:03 -0700 PDT $
# ----------
# An attempt to count the number of specs in the system each day, based on spec-depot files. 
# Requires four parameters - P4PORT, a depot path (for example, '//spec/user/...') and two dates in the format 'yyyy/mm/dd'.
#
# Drops out on failing one of various checks
# 1. wrong number of parameters
# 2. no 'p4' available
# 3. no valid login ticket
# 4. can't find path specified
# 5. log file already exists (might not want to overwrite the content)
# 6. startdate invalid
# 7. enddate invalid
#
# Once through, the script runs 'p4 files' against the specs in the spec depot. 
# It reports the count on a given day (specs not deleted), then number added, edited and deleted.
# ----------
# Notes: 
# - 'p4 files @date,date' may not be the quickest command to run - performance improvements may be possible;
# - this does not distinguish between specs of different types, just counts the specs it finds;
#   - in particular, no distinction is made between standard, operator and service users;
# - some specs may not be present in the spec depot (spec depot created after them, perhaps);
# - the SpecMap field in the spec depot spec may exclude other specs. 
# 
# ==========

# expecting four parameters - P4PORT, a depot path and two dates
if [ "$#" -ne 4 ]; then 
    printf "Usage: %s %s %s %s %s\n%s\n%s\n" $0 "P4PORT" "depotpath" "date" "date" "depot: //spec/user/..." "date format is 'yyyy/mm/dd'" 
    exit 1
fi

# check there is a 'p4' available
p4cli=`which p4` 
if [ "$?" -ne 0 ]; then 
    printf "%s: 'p4' client not found.\n" $0
    exit 2
fi
 
# check there's a valid connection (looks for ticket)
$p4cli login -s > /dev/null 2>&1
if [ $? -ne 0 ]; then 
    printf "%s: 'p4' not logged in or no connection available.\n" $0
    exit 3
fi

p4cmd="$p4cli -p $1 files"
depotpath=$2

# valid spec type?
##spectype=$2
##case $spectype in
##    branch|client|depot|group|job|label|ldap|protects|remote|server|stream|trigger|typemap|user) ;;
##    *) printf "%s: Not a valid spec type.\n" $0; exit 4 ;;
##esac

# check if the path exists/can be accessed
$p4cmd -m1 $depotpath > /dev/null 
if [ $? -ne 0 ]; then
    printf "%s: path %s does not exist or is not accessible.\n" $0 $depotpath
    exit 4
fi

# name the output file using the script's name
of="$(basename $0).log"

if [ -f $of ]; then
    printf "%s: log file '%s' already exists. Not continuing.\n" $0 $of
    exit 5
fi

sd=$3
ed=$4

startdate=$( date -d $sd +"%Y/%m/%d" ) || exit 6
enddate=$( date -d $ed +"%Y/%m/%d" ) || exit 7

d="$startdate"
D=$( date -d "$d + 1 day" +"%Y/%m/%d" )

#open file descriptor 3 for read-write on the log file
exec 3<> $of

printf "$0: counting spec changes in the spec depot\n" >&3
printf "Parameters: %s %s %s %s\n" $1 $2 $3 $4 >&3
printf "Date range: %s - %s\n" $startdate $enddate >&3

while [ "$( date -d "$d" +%Y%m%d)" -le "$( date -d "$enddate" +%Y%m%d)" ]; do
    uc=$( $p4cmd $depotpath@$d     2>/dev/null | grep -v "delete default change" | wc -l )
    ua=$( $p4cmd $depotpath@$d,@$D 2>/dev/null | grep    "add default change"    | wc -l )
    ue=$( $p4cmd $depotpath@$d,@$D 2>/dev/null | grep    "edit default change"   | wc -l )
    ud=$( $p4cmd $depotpath@$d,@$D 2>/dev/null | grep    "delete default change" | wc -l )

    printf "%s-%s\tc: %d\ta: %d\te: %d\td: %d\n" $d $D $uc $ua $ue $ud >&3
    d=$D
    D=$( date -d "$d + 1 day" +"%Y/%m/%d" )
done

printf "$0: end of count.\n" >&3

# close file descriptor 3
exec 3>&-
